import dataclasses as dc

from armarx_skills.manager import dto
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)
from armarx_memory.ice_conv.ice_converter import IceConverter


@dc.dataclass
class Edge:
    """
    A class representing an edge for a Fluxio composite skill.

    Attributes:
    - fromNodeId: Identificator - the ID of the node where the edge starts
    - fromParameterId: Identificator - the ID of the parameter at the starting node
    - toNodeId: Identificator - the ID of the node where the edge ends
    - toParameterId: Identificator - the ID of the parameter at the ending node
    """

    fromNodeId: Identificator
    fromParameterId: Identificator
    toNodeId: Identificator
    toParameterId: Identificator


class FluxioEdgeConv(IceConverter):
    """
    A class representing a Fluxio Edge Converter.

    This class inherits from IceConverter and is used to convert data from Fluxio Edge devices.

    Attributes:
    - None

    Methods:
    - __init__(): Initializes the FluxioEdgeConv object.
    - _import_dto(): Imports the DTO class for the converter.
    - _from_ice(): Converts the DTO to the Edge object.
    - _to_ice(): Converts the Edge object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioEdgeConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This function imports and returns the DTO (Data Transfer Object) class for FluxioEdge.

        Parameters:
        cls (class): The class for which the DTO is being imported.

        Return:
        class: The DTO class for FluxioEdge.

        Raises:
        None
        """

        return dto.FluxioEdge

    def _from_ice(self, dt: dto.FluxioEdge) -> Edge:
        """
        Converts a dto.FluxioEdge object to an Edge object.

        Parameters:
        dt (dto.FluxioEdge): The dto.FluxioEdge object to convert.

        Return:
        Edge: The converted Edge object.

        Raises:
        None
        """
        return Edge(
            fromNodeId=FluxioIdentificatorConv().from_ice(dt.fromNodeId),
            fromParameterId=FluxioIdentificatorConv().from_ice(dt.fromParameterId),
            toNodeId=FluxioIdentificatorConv().from_ice(dt.toNodeId),
            toParameterId=FluxioIdentificatorConv().from_ice(dt.toParameterId),
        )

    def _to_ice(self, bo: Edge) -> dto.FluxioEdge:
        """
        Converts a business object Edge to a data transfer object FluxioEdge.

        Parameters:
        bo (Edge): The business object Edge to be converted.

        Return:
        dto.FluxioEdge: The converted data transfer object FluxioEdge.

        Raises:
        None
        """

        return dto.FluxioEdge(
            fromNodeId=FluxioIdentificatorConv().to_ice(bo.fromNodeId),
            fromParameterId=FluxioIdentificatorConv().to_ice(bo.fromParameterId),
            toNodeId=FluxioIdentificatorConv().to_ice(bo.toNodeId),
            toParameterId=FluxioIdentificatorConv().to_ice(bo.toParameterId),
        )

