import dataclasses as dc

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter


@dc.dataclass
class Identificator:
    """
    A class representing an Identificator object.

    Attributes:
    id (str): The unique identifier for the object.
    hint (str): A hint or description for the object.

    Methods:
    None

    Example:
    >>> ident = Identificator(id="12345", hint="This is a sample identificator")
    """

    id: str
    hint: str


class FluxioIdentificatorConv(IceConverter):
    """
    A class for converting identifiers in the Fluxio system.

    This class inherits from IceConverter and provides methods for converting identifiers used in the Fluxio system.

    Attributes:
    - None

    Methods:
    - __init__: Initializes the FluxioIdentificatorConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the Identificator object.
    - _to_ice: Converts the Identificator object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioIdentificatorConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioIdentificator data transfer object (DTO).

        Return:
        dto.FluxioIdentificator: The FluxioIdentificator data transfer object.

        Raises:
        None
        """
        return dto.FluxioIdentificator

    def _from_ice(self, dt: dto.FluxioIdentificator) -> Identificator:
        """
        Converts a FluxioIdentificator object to an Identificator object.

        Parameters:
        dt (dto.FluxioIdentificator): The FluxioIdentificator object to convert.

        Return:
        Identificator: The converted Identificator object.

        Raises:
        None
        """
        return Identificator(id=dt.id, hint=dt.hint)

    def _to_ice(self, bo: Identificator) -> dto.FluxioIdentificator:
        """
        Converts an Identificator object to a FluxioIdentificator object.

        Parameters:
        bo (Identificator): The Identificator object to be converted.

        Return:
        dto.FluxioIdentificator: The converted FluxioIdentificator object.

        Raises:
        None
        """
        return dto.FluxioIdentificator(id=bo.id, hint=bo.hint)
