import dataclasses as dc
from enum import Enum

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter


class NodeTypes(str, Enum):
    SUBSKILL = "SUBSKILL"
    PARAMETER = "PARAMETER"
    CONTROL = "CONTROL"


@dc.dataclass
class Node:
    """
    A class representing a node in a Fluxio compositer skill.

    Attributes:
    - nodeType (NodeTypes): The type of the node.
    - nodeId (str): The unique identifier of the node.
    - xPos (float): The x-coordinate of the node.
    - yPos (float): The y-coordinate of the node.
    """

    nodeType: NodeTypes
    nodeId: str
    xPos: float
    yPos: float


class FluxioNodeConv(IceConverter):
    """
    A class representing a Fluxio Node Converter.

    This class extends the IceConverter class and is used to convert data from Fluxio nodes.

    Attributes:
    - None

    Methods:
    - __init__: Initializes the FluxioNodeConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the Node object.
    - _to_ice: Converts the Node object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioNodeConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioNode data transfer object (DTO).

        Return:
        dto.FluxioNode: The FluxioNode data transfer object.

        Raises:
        None
        """
        return dto.FluxioNode

    def _from_ice(self, dt: dto.FluxioNode) -> Node:
        """
        Converts a FluxioNode object to a Node object.

        Parameters:
        dt (dto.FluxioNode): The FluxioNode object to convert.

        Return:
        Node: The converted Node object.

        Raises:
        None
        """
        return Node(
            nodeType=NodeTypes(dt.nodeType),
            nodeId=dt.nodeId,
            xPos=dt.xPos,
            yPos=dt.yPos,
        )

    def _to_ice(self, bo: Node) -> dto.FluxioNode:
        """
        Converts a Node object to a FluxioNode object.

        Parameters:
        bo (Node): The Node object to be converted.

        Return:
        dto.FluxioNode: The converted FluxioNode object.

        Raises:
        None
        """
        return dto.FluxioNode(
            nodeType=bo.nodeType, nodeId=bo.nodeId, xPos=bo.xPos, yPos=bo.yPos
        )
