import dataclasses as dc
from typing import List

from armarx_skills.manager import dto
from armarx_skills.manager.fluxio_value import Value, FluxioValueConv
from armarx_skills.manager.fluxio_type_identificator import TypeIdentificator, FluxioTypeIdentificatorConv
from armarx_memory.ice_conv.ice_converter import IceConverter


@dc.dataclass
class Parameter:
    """
    A class representing a parameter of a Fluxio skill.

    Attributes:
    - id (str): The unique identifier for the parameter.
    - name (str): The name of the parameter.
    - description (str): A description of the parameter.
    - type (TypeIdentificator): The type of the parameter.
    - required (bool): Indicates whether the parameter is required.
    - isInput (bool): Indicates whether the parameter is an input parameter.
    - values (List[Value]): A list of possible values for the parameter.
    """

    id: str
    name: str
    description: str
    # type: str  # TODO: Update to a more specific type.
    type: TypeIdentificator
    required: bool
    isInput: bool
    values: List[Value]


class FluxioParameterConv(IceConverter):
    """
    A class for converting parameters in Fluxio format to Ice format.

    This class inherits from IceConverter and provides methods for converting parameters in Fluxio format to Ice format.

    Attributes:
    - None

    Methods:
    - __init__(): Initializes the FluxioParameterConv class.
    - _import_dto(): Imports the DTO class for the converter.
    - _from_ice(): Converts the DTO to the Parameter object.
    - _to_ice(): Converts the Parameter object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioParameterConv class.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioParameter data transfer object (DTO).

        Return:
        dto.FluxioParameter: The FluxioParameter data transfer object.

        Raises:
        None
        """
        return dto.FluxioParameter

    def _from_ice(self, dt: dto.FluxioParameter) -> Parameter:
        """
        Converts a FluxioParameter object to a Parameter object.

        Parameters:
            dt (dto.FluxioParameter): The FluxioParameter object to convert.

        Return:
            Parameter: The converted Parameter object.

        Raises:
            None
        """
        return Parameter(
            id=dt.id,
            name=dt.name,
            description=dt.description,
            type=FluxioTypeIdentificatorConv().from_ice(dt.type),
            required=dt.required,
            isInput=dt.isInput,
            values=[FluxioValueConv().from_ice(v) for v in dt.values],
        )

    def _to_ice(self, bo: Parameter) -> dto.FluxioParameter:
        """
        Converts a Parameter object to a FluxioParameter object.

        Parameters:
        bo (Parameter): The Parameter object to be converted.

        Return:
        dto.FluxioParameter: The converted FluxioParameter object.

        Raises:
        None
        """
        return dto.FluxioParameter(
            id=bo.id,
            name=bo.name,
            description=bo.description,
            type=FluxioTypeIdentificatorConv().to_ice(bo.type),
            required=bo.required,
            isInput=bo.isInput,
            values=[FluxioValueConv().to_ice(v) for v in bo.values],
        )
