import dataclasses as dc

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_skills.manager.fluxio_node import Node, NodeTypes
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)


@dc.dataclass
class ParameterNode(Node):
    """
    A class representing a Parameter Node in a Fluxio compositer skill.

    Attributes:
    - nodeType (str): The type of the node.
    - nodeId (str): The unique identifier of the node.
    - xPos (float): The x-coordinate of the node.
    - yPos (float): The y-coordinate of the node.
    - parameterId (Identificator): The identifier of the parameter associated with the node.
    """

    parameterId: Identificator

    def __init__(
        self,
        nodeType: NodeTypes,
        nodeId: str,
        xPos: float,
        yPos: float,
        parameterId: Identificator,
    ) -> None:
        """
        Initialize a ParameterNode object.

        Parameters:
        - nodeType (str): The type of the node.
        - nodeId (str): The unique identifier of the node.
        - xPos (float): The x-coordinate of the node.
        - yPos (float): The y-coordinate of the node.
        - parameterId (Identificator): The identifier of the parameter associated with the node.
        """
        super().__init__(nodeType, nodeId, xPos, yPos)
        self.parameterId = parameterId


class FluxioParameterNodeConv(IceConverter):
    """
    A class representing a Fluxio Parameter Node Converter.

    This class inherits from IceConverter and is used to convert Fluxio Parameter Nodes.

    Attributes:
    - None

    Methods:
    - __init__: Initializes the FluxioParameterNodeConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the ParameterNode object.
    - _to_ice: Converts the ParameterNode object to the DTO.
    """

    def __init__(self):
        """
        Initializes the FluxioParameterNodeConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioNode data transfer object (DTO).

        Return:
        dto.FluxioNode: The FluxioNode data transfer object.

        Raises:
        None
        """
        return dto.FluxioNode

    def _from_ice(self, dt: dto.FluxioNode) -> ParameterNode:
        """
        Converts a dto.FluxioNode object to a ParameterNode object.

        Parameters:
        dt (dto.FluxioNode): The dto.FluxioNode object to convert.

        Returns:
        ParameterNode: The converted ParameterNode object.

        Raises:
        None
        """
        return ParameterNode(
            nodeType=NodeTypes(dt.nodeType),
            nodeId=dt.nodeId,
            xPos=dt.xPos,
            yPos=dt.yPos,
            parameterId=FluxioIdentificatorConv().from_ice(dt.parameterId),
        )

    def _to_ice(self, bo: ParameterNode) -> dto.FluxioNode:
        """
        Converts a ParameterNode object to a FluxioNode object.

        Parameters:
        bo (ParameterNode): The ParameterNode object to be converted.

        Returns:
        dto.FluxioNode: The converted FluxioNode object.

        Raises:
        None
        """
        return dto.FluxioNode(
            nodeType=bo.nodeType,
            nodeId=bo.nodeId,
            xPos=bo.xPos,
            yPos=bo.yPos,
            parameterId=FluxioIdentificatorConv().to_ice(bo.parameterId),
        )
