import dataclasses as dc

from armarx_skills.manager import dto
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)
from armarx_memory.ice_conv.ice_converter import IceConverter


@dc.dataclass
class Profile:
    """
    A class representing a settings profile for skill parameter values.

    Attributes:
    - id (str): The unique identifier for the profile.
    - name (str): The name of the user.
    - description (str): A description of the user.
    - parentId (Identificator | None): The identifier of the parent profile, if any.
    """

    id: str
    name: str
    description: str
    parentId: Identificator | None = None


class FluxioProfileConv(IceConverter):
    """
    A class for converting Fluxio profiles.

    This class extends IceConverter and provides methods for converting Fluxio profiles.

    Attributes:
    - fluxio_identificator_conv: An instance of FluxioIdentificatorConv used for converting Fluxio identifiers.

    Methods:
    - __init__: Initializes the FluxioProfileConv class and sets up the fluxio_identificator_conv attribute.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the Profile object.
    - _to_ice: Converts the Profile object to the DTO.
    """

    def __init__(self):
        """
        Initializes the FluxioProfileConv class.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()
        self.fluxio_identificator_conv = FluxioIdentificatorConv()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioProfile data transfer object (DTO) class.

        Return:
        dto.FluxioProfile: The FluxioProfile data transfer object class.

        Raises:
        None
        """
        return dto.FluxioProfile

    def _from_ice(self, dt: dto.FluxioProfile) -> Profile:
        """
        Converts a dto.FluxioProfile object to a Profile object.

        Parameters:
        dt (dto.FluxioProfile): The dto.FluxioProfile object to be converted.

        Return:
        Profile: The converted Profile object.

        Raises:
        None
        """
        return Profile(
            id=dt.id,
            name=dt.name,
            description=dt.description,
            parentId=None
            if (not dt.parentIdHasValue)
            else self.fluxio_identificator_conv.from_ice(dt.parentId),
        )

    def _to_ice(self, bo: Profile) -> dto.FluxioProfile:
        """
        Converts a Profile object to a FluxioProfile object.

        Parameters:
        bo (Profile): The Profile object to be converted.

        Return:
        dto.FluxioProfile: The converted FluxioProfile object.

        Raises:
        None

        If the parentId attribute of the Profile object is None, parentIdHasValue is set to False and parentId is set to an empty dictionary.
        If the parentId attribute is not None, parentIdHasValue is set to True and parentId is converted using the fluxio_identificator_conv.to_ice method.

        """
        return dto.FluxioProfile(
            id=bo.id,
            name=bo.name,
            description=bo.description,
            parentIdHasValue=False if (bo.parentId is None) else True,
            parentId={"id": "", "hint": ""}
            if (bo.parentId is None)
            else self.fluxio_identificator_conv.to_ice(bo.parentId),
        )
