import dataclasses as dc
from typing import List

from armarx_skills.manager import dto
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)
from armarx_skills.manager.fluxio_parameter import Parameter, FluxioParameterConv
from armarx_skills.manager.fluxio_node import Node, FluxioNodeConv, NodeTypes
from armarx_skills.manager.fluxio_parameter_node import (
    ParameterNode,
    FluxioParameterNodeConv,
)
from armarx_skills.manager.fluxio_subskill_node import (
    SubSkillNode,
    FluxioSubSkillNodeConv,
)
from armarx_skills.manager.fluxio_control_node import ControlNode, FluxioControlNodeConv
from armarx_skills.manager.fluxio_edge import Edge, FluxioEdgeConv
from armarx_memory.ice_conv.ice_converter import IceConverter

from armarx_core.time.duration import Duration, DurationIceConv


@dc.dataclass
class Skill:
    """
    Represents a Fluxio skill entity.

    Attributes:
    - id (str): The unique identifier of the skill.
    - name (str): The name of the skill.
    - description (str): A brief description of the skill.
    - timeout (Duration): The timeout of the skill.
    - lastChanged (str): The timestamp when the skill was last modified.
    - executable (bool): Indicates if the skill is executable.
    - native (bool): Indicates if the skill is native and not composite.
    - skillProviderId (Identificator): The identifier of the skill provider.
    - parameters (List[Parameter]): A list of parameters required by the skill.
    - nodes (List[Node], optional): A list of nodes associated with the skill.
    - edges (List[Edge], optional): A list of edges associated with the skill.
    """

    id: str
    name: str
    description: str
    timeout: int
    lastChanged: str
    executable: bool
    native: bool
    skillProviderId: Identificator
    parameters: List[Parameter]
    nodes: List[Node] | None = None
    edges: List[Edge] | None = None


class FluxioSkillConv(IceConverter):
    """
    A class for converting Fluxio skills to Ice skills.

    This class inherits from IceConverter and provides methods for converting Fluxio skills to Ice skills.

    Attributes:
    - None

    Methods:
    - __init__: Initializes the FluxioSkillConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the Skill object.
    - _to_ice: Converts the Skill object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioSkillConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()
        self.duration_conv = DurationIceConv()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioSkill data transfer object (DTO).

        Return:
        dto.FluxioSkill: The FluxioSkill data transfer object.

        Raises:
        None
        """
        return dto.FluxioSkill

    def _from_ice(self, dt: dto.FluxioSkill) -> Skill:
        """
        Converts a dto.FluxioSkill object to a Skill object.

        Parameters:
        dt (dto.FluxioSkill): The dto.FluxioSkill object to be converted.

        Return:
        Skill: The converted Skill object.

        Raises:
        None
        """
        ret = Skill(
            id=dt.id,
            name=dt.name,
            description=dt.description,
            timeout=self.duration_conv.from_ice(dt.timeout).toMilliSeconds(),
            lastChanged=dt.lastChanged,
            executable=dt.executable,
            native=dt.native,
            skillProviderId=FluxioIdentificatorConv().from_ice(dt.skillProviderId),
            parameters=[FluxioParameterConv().from_ice(p) for p in dt.parameters],
            nodes=[],
            edges=[FluxioEdgeConv().from_ice(e) for e in dt.edges]
            if dt.edgesHasValue is True
            else [],
        )

        if dt.nodesHasValue is True:
            for n in dt.nodes:
                if n.nodeType == NodeTypes.PARAMETER:
                    ret.nodes.append(FluxioParameterNodeConv().from_ice(n))
                elif n.nodeType == NodeTypes.SUBSKILL:
                    ret.nodes.append(FluxioSubSkillNodeConv().from_ice(n))
                elif n.nodeType == NodeTypes.CONTROL:
                    ret.nodes.append(FluxioControlNodeConv().from_ice(n))
                else:
                    ret.nodes.append(FluxioNodeConv().from_ice(n))
        return ret

    def _to_ice(self, bo: Skill) -> dto.FluxioSkill:
        """
        Converts a Skill business object to a FluxioSkill data transfer object.

        Parameters:
        bo (Skill): The Skill business object to be converted.

        Return:
        dto.FluxioSkill: The converted FluxioSkill data transfer object.

        Raises:
        None
        """
        ret = dto.FluxioSkill(
            id=bo.id,
            name=bo.name,
            description=bo.description,
            timeout=self.duration_conv.to_ice(Duration(microSeconds=bo.timeout * 1000)),
            lastChanged=bo.lastChanged,
            executable=bo.executable,
            native=bo.native,
            skillProviderId=FluxioIdentificatorConv().to_ice(bo.skillProviderId),
            parameters=[FluxioParameterConv().to_ice(p) for p in bo.parameters],
            nodesHasValue=False if (bo.nodes is None) else True,
            nodes=[],
            edgesHasValue=False if (bo.edges is None) else True,
            edges=[]
            if (bo.edges is None)
            else [FluxioEdgeConv().to_ice(e) for e in bo.edges],
        )
        if bo.nodes is not None:
            for n in bo.nodes:
                if isinstance(n, ParameterNode):
                    ret.nodes.append(FluxioParameterNodeConv().to_ice(n))
                elif isinstance(n, SubSkillNode):
                    ret.nodes.append(FluxioSubSkillNodeConv().to_ice(n))
                elif isinstance(n, ControlNode):
                    ret.nodes.append(FluxioControlNodeConv().to_ice(n))
                else:
                    ret.nodes.append(FluxioNodeConv().to_ice(n))
        return ret
