import dataclasses as dc

from armarx_skills.manager import dto
from armarx_core.time.date_time import DateTimeIceConverter
from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_skills.core.execution_status import ExecutionStatus, ExecutionStatusConv
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)


@dc.dataclass
class SkillStatusUpdate:
    """
    A class representing an update to the status of a skill execution.

    Attributes:
    timestamp (int): The timestamp of the status update. Generated by armarx::DateTime::Now().
    executionId (str): The unique identifier of the skill execution.
    subSkillNodeId (Identificator): The identifier of the sub-skill node. If composite skill then the id is the skill id itself.
    status (ExecutionStatus): The status of the skill execution.
    """

    timestamp: int
    executionId: str
    subSkillNodeId: Identificator
    status: ExecutionStatus


class FluxioSkillStatusUpdateConv(IceConverter):
    """
    A class for converting Fluxio skill status updates.

    This class inherits from IceConverter and provides methods for converting Fluxio skill status updates.

    Attributes:
    - date_time_conv (DateTimeIceConverter): A converter for date and time values.
    - status_conv (ExecutionStatusConv): A converter for execution status values.
    - fluxio_identificator_conv (FluxioIdentificatorConv): A converter for Fluxio identificator values.
    """

    def __init__(self):
        """
        Initializes the FluxioSkillStatusUpdateConv object.

        This method initializes the converters for date and time values, execution status values, and Fluxio identificator values.
        """
        super().__init__()
        self.date_time_conv = DateTimeIceConverter()
        self.status_conv = ExecutionStatusConv()
        self.fluxio_identificator_conv = FluxioIdentificatorConv()

    @classmethod
    def _import_dto(cls):
        """
        Returns the data transfer object (DTO) class for FluxioSkillStatusUpdate.

        Return:
            dto.FluxioSkillStatusUpdate: The class representing the data transfer object for FluxioSkillStatusUpdate.
        """
        return dto.FluxioSkillStatusUpdate

    def _from_ice(self, dt: dto.FluxioSkillStatusUpdate) -> SkillStatusUpdate:
        """
        Converts a dto.FluxioSkillStatusUpdate object to a SkillStatusUpdate object.

        Parameters:
        dt (dto.FluxioSkillStatusUpdate): The dto object to be converted.

        Return:
        SkillStatusUpdate: The converted SkillStatusUpdate object.

        Raises:
        None
        """
        return SkillStatusUpdate(
            timestamp=self.date_time_conv.from_ice(dt.timestamp),
            executionId=dt.executionId,
            subSkillNodeId=self.fluxio_identificator_conv.from_ice(dt.subSkillNodeId),
            status=self.status_conv.from_ice(dt.status),
        )

    def _to_ice(self, bo: SkillStatusUpdate) -> dto.FluxioSkillStatusUpdate:
        """
        Converts a SkillStatusUpdate object to a FluxioSkillStatusUpdate object.

        Parameters:
        bo (SkillStatusUpdate): The SkillStatusUpdate object to be converted.

        Return:
        FluxioSkillStatusUpdate: The converted FluxioSkillStatusUpdate object.

        Raises:
        None
        """

        return dto.FluxioSkillStatusUpdate(
            timestamp=self.date_time_conv.to_ice(bo.timestamp),
            executionId=bo.executionId,
            subSkillNodeId=self.fluxio_identificator_conv.to_ice(bo.subSkillNodeId),
            status=self.status_conv.to_ice(bo.status),
        )
