import dataclasses as dc

from armarx_skills.manager import dto
from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_skills.manager.fluxio_node import Node, NodeTypes
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)


@dc.dataclass
class SubSkillNode(Node):
    """
    A class representing a node in a composite skill that contains a sub-skill.

    Attributes:
    - skillId: Identificator - The identifier of the sub-skill associated with this node

    """

    skillId: Identificator

    def __init__(
        self,
        nodeType: NodeTypes,
        nodeId: str,
        xPos: float,
        yPos: float,
        skillId: Identificator,
    ):
        """
        Initialize a Node object with the given parameters.

        Parameters:
        nodeType (NodeTypes): The type of the node.
        nodeId (str): The unique identifier of the node.
        xPos (float): The x-coordinate position of the node.
        yPos (float): The y-coordinate position of the node.
        skillId (Identificator): The identifier of the skill associated with the node.

        Return:
        None

        Raises:
        None
        """
        super().__init__(nodeType, nodeId, xPos, yPos)
        self.skillId = skillId


class FluxioSubSkillNodeConv(IceConverter):
    """
    A class representing a Fluxio SubSkill Node Converter.

    This class inherits from IceConverter and is used to convert Fluxio SubSkill Nodes.

    Attributes:
    - None

    Methods:
    - __init__: Initializes the FluxioSubSkillNodeConv object.
    - _import_dto: Imports the DTO class for the converter.
    - _from_ice: Converts the DTO to the SubSkillNode object.
    - _to_ice: Converts the SubSkillNode object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioSubSkillNodeConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioNode data transfer object (DTO).

        Return:
        dto.FluxioNode: The FluxioNode data transfer object.

        Raises:
        None
        """
        return dto.FluxioNode

    def _from_ice(self, dt: dto.FluxioNode) -> SubSkillNode:
        """
        Converts a dto.FluxioNode object to a SubSkillNode object.

        Parameters:
        dt (dto.FluxioNode): The FluxioNode object to convert.

        Return:
        SubSkillNode: The converted SubSkillNode object.

        Raises:
        None
        """
        return SubSkillNode(
            nodeType=NodeTypes(dt.nodeType),
            nodeId=dt.nodeId,
            xPos=dt.xPos,
            yPos=dt.yPos,
            skillId=FluxioIdentificatorConv().from_ice(dt.skillId),
        )

    def _to_ice(self, bo: SubSkillNode) -> dto.FluxioNode:
        """
        Converts a SubSkillNode object to a FluxioNode object.

        Parameters:
        bo (SubSkillNode): The SubSkillNode object to be converted.

        Returns:
        dto.FluxioNode: The converted FluxioNode object.

        Raises:
        None
        """

        return dto.FluxioNode(
            nodeType=bo.nodeType,
            nodeId=bo.nodeId,
            xPos=bo.xPos,
            yPos=bo.yPos,
            skillId=FluxioIdentificatorConv().to_ice(bo.skillId),
        )
