import warnings

from armarx_memory.aron.aron_ice_types import AronTypeIceTypes

from .fluxio_types import (
    FluxioBasicType,
    FluxioDoubleGenericType,
    FluxioGenericType,
    FluxioObjectType,
    FluxioTypeIdentifier,
    FluxioEnumType,
)


class FluxioTypeAronConverter:
    @classmethod
    def aron_type_to_fluxio_type(
        cls, aron_type: AronTypeIceTypes.GenericType
    ) -> (
        FluxioBasicType | FluxioGenericType | FluxioDoubleGenericType | FluxioObjectType
    ):
        ret = None
        maybe = aron_type.maybe != AronTypeIceTypes._maybe["NONE"]
        if isinstance(aron_type, AronTypeIceTypes.Int):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="Integer", description="Integer", default_value=defaultValue
            )            
        elif isinstance(aron_type, AronTypeIceTypes.Long):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="Long", description="Long", default_value=defaultValue
            )
        elif isinstance(aron_type, AronTypeIceTypes.Float):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="Float", description="Float", default_value=defaultValue
            )
        elif isinstance(aron_type, AronTypeIceTypes.Double):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="Double", description="Double", default_value=defaultValue
            )
        elif isinstance(aron_type, AronTypeIceTypes.Bool):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="Boolean", description="Boolean", default_value=defaultValue
            )
        elif isinstance(aron_type, AronTypeIceTypes.String):
            defaultValue = aron_type.defaultValue
            ret = FluxioBasicType(
                name="String", description="String", default_value=defaultValue
            )
        elif isinstance(aron_type, AronTypeIceTypes.List):
            generic = cls.aron_type_to_fluxio_type(aron_type.acceptedType)
            ret = FluxioGenericType(
                name="List",
                description=f"List with elements of type '{generic.name}'",
                generic=generic,
                optional=maybe,
                max_number_of_elements=0,
            )
        elif isinstance(aron_type, AronTypeIceTypes.Quaternion):
            elementType = str(aron_type.elementType)
            generic = FluxioBasicType(
                name=elementType,
                description=elementType,
            )
            description = f"Quaternion with elements of type '{generic.name}'"
            default = aron_type.defaultValue
            ret = FluxioGenericType(
                name="Quaternion",
                description=description,
                generic=generic,
                max_number_of_elements=4,
                optional=maybe,
                default_value=default,
            )
        elif isinstance(aron_type, AronTypeIceTypes.Dict):
            generic1 = FluxioBasicType(name="String", description="String")
            generic2 = cls.aron_type_to_fluxio_type(aron_type.acceptedType)
            ret = FluxioDoubleGenericType(
                name="Dict",
                description="Dictionary with keys of type 'String'"
                + " and values of type '"
                + generic2.name
                + "'",
                generic1=generic1,
                generic2=generic2,
            )
        elif isinstance(aron_type, AronTypeIceTypes.Object):
            if aron_type.objectName == "Event":
                ret = FluxioBasicType(name="Event", description="Event")
            else:
                parent = (
                    cls.aron_type_to_fluxio_type(aron_type.parent)
                    if aron_type.parent
                    else None
                )
                ret = FluxioObjectType(
                    name=aron_type.objectName,
                    type_identifier=FluxioTypeIdentifier.OBJECT,
                    parent=parent,
                )
                for k, v in aron_type.elementTypes.items():
                    child = cls.aron_type_to_fluxio_type(v)
                    ret.add_attribute(name=k, type=child)
        elif isinstance(aron_type, AronTypeIceTypes.Matrix):
            rows = aron_type.rows
            cols = aron_type.cols
            mat_type = str(aron_type.elementType)
            description = (
                f"Matrix with dimensions {rows}x{cols} and elements of type {mat_type}"
            )
            generic = FluxioBasicType(name=mat_type, description=mat_type)
            inner_type = FluxioGenericType(
                name="row",
                description="",
                max_number_of_elements=rows,
                generic=generic,
            )
            ret = FluxioGenericType(
                name="Matrix",
                description=description,
                max_number_of_elements=cols,
                generic=inner_type,
            )
        elif isinstance(aron_type, AronTypeIceTypes.IntEnum):
            ret = FluxioEnumType(
                name=aron_type.enumName, description=aron_type.enumName
            )
            for k, v in aron_type.acceptedValues.items():
                ret.add_attribute(name=k, value=v)
        else:
            # TODO: this won't work because aron_type can't be converted to a strings
            message = f"Unknown type: {aron_type}"
            warnings.warn(message=message, category=RuntimeWarning)
            raise Exception(message)
            # return ret
        ret.optional = maybe
        return ret

    @classmethod
    def fluxio_type_to_aron(
        cls,
        fluxio_type: (
            FluxioBasicType
            | FluxioGenericType
            | FluxioDoubleGenericType
            | FluxioObjectType
        ),
    ) -> any:
        ret = None
        identifier = fluxio_type.type_identifier
        if identifier == FluxioTypeIdentifier.BASIC:
            ret = cls.__fluxio_basic_type_to_aron(fluxio_type)
        elif identifier == FluxioTypeIdentifier.GENERIC:
            ret = cls.__fluxio_generic_type_to_aron(fluxio_type)
        elif identifier == FluxioTypeIdentifier.DOUBLE_GENERIC:
            ret = cls.__fluxio_double_generic_type_to_aron(fluxio_type)
        elif identifier == FluxioTypeIdentifier.OBJECT:
            ret = cls.__fluxio_obj_type_to_aron(fluxio_type)
        elif identifier == FluxioTypeIdentifier.ENUM:
            ret = cls.__fluxio_enum_type_to_aron(fluxio_type)
        else:
            warnings.warn(
                message="Unknown type identifier: " + identifier,
                category=RuntimeWarning,
            )
            return ret
        return ret

    @classmethod
    def __fluxio_basic_type_to_aron(
        cls, fluxio_type: FluxioBasicType
    ) -> AronTypeIceTypes.GenericType:
        ret = None
        maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
        if fluxio_type.name == "Integer":
            ret = AronTypeIceTypes.int(default_value=fluxio_type.default_value)
        elif fluxio_type.name == "Event":
            maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
            ret = AronTypeIceTypes.object(
                object_name="Event", element_types={}, maybe=maybe,
            )
        elif fluxio_type.name == "Long":
            ret = AronTypeIceTypes.long(default_value=fluxio_type.default_value)
        elif fluxio_type.name == "Float":
            ret = AronTypeIceTypes.float(default_value=fluxio_type.default_value)
        elif fluxio_type.name == "Double":
            ret = AronTypeIceTypes.double(default_value=fluxio_type.default_value)
        elif fluxio_type.name == "Boolean":
            ret = AronTypeIceTypes.bool(default_value=fluxio_type.default_value)
        elif fluxio_type.name == "String":
            ret = AronTypeIceTypes.string(default_value=fluxio_type.default_value)
        else:
            warnings.warn(
                message="Unknown basic type: " + fluxio_type.name,
                category=RuntimeWarning,
            )
            return ret
        ret.maybe = AronTypeIceTypes._maybe[maybe]
        return ret

    @classmethod
    def __fluxio_generic_type_to_aron(
        cls, fluxio_type: FluxioGenericType
    ) -> AronTypeIceTypes.GenericType:
        ret = None
        maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
        if fluxio_type.name == "List":
            generic = cls.fluxio_type_to_aron(fluxio_type.generic)
            ret = AronTypeIceTypes.list(accepted_type=generic, maybe=maybe)
        elif fluxio_type.name == "Quaternion":
            ret = AronTypeIceTypes.quaternion(
                element_type=fluxio_type.generic.name,
                maybe=maybe,
                default_value=fluxio_type.default_value or "",
            )
        elif fluxio_type.name == "Matrix":
            rows = fluxio_type.generic.max_number_of_elements
            cols = fluxio_type.max_number_of_elements
            mat_type = fluxio_type.generic.generic.name
            maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
            ret = AronTypeIceTypes.matrix(
                rows=rows, cols=cols, element_type=mat_type, maybe=maybe
            )
        return ret

    @classmethod
    def __fluxio_double_generic_type_to_aron(
        cls,
        fluxio_type: FluxioDoubleGenericType,
    ) -> AronTypeIceTypes:
        ret = None
        if fluxio_type.name == "Dict":
            generic2 = cls.fluxio_type_to_aron(fluxio_type.generic2)
            maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
            ret = AronTypeIceTypes.dict(accepted_type=generic2, maybe=maybe)
        return ret

    @classmethod
    def __fluxio_obj_type_to_aron(
        cls, fluxio_type: FluxioObjectType
    ) -> AronTypeIceTypes.GenericType:
        atr = {}
        for attribute in fluxio_type.attributes:
            atr[attribute.name] = cls.fluxio_type_to_aron(attribute.type)
        maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
        parent = (
            cls.fluxio_type_to_aron(fluxio_type.parent) if fluxio_type.parent else None
        )
        ret = AronTypeIceTypes.object(
            object_name=fluxio_type.name, element_types=atr, maybe=maybe, parent=parent
        )
        return ret

    @classmethod
    def __fluxio_enum_type_to_aron(
        cls, fluxio_type: FluxioEnumType
    ) -> AronTypeIceTypes.GenericType:
        maybe = "OPTIONAL" if fluxio_type.optional else "NONE"
        ret = AronTypeIceTypes.int_enum(name=fluxio_type.name, maybe=maybe)
        dict = {}
        for k, v in fluxio_type.attributes.items():
            dict[k] = v
        ret.acceptedValues = dict
        return ret
