from typing import Optional
import dataclasses as dc
from enum import Enum

from armarx_skills.manager.fluxio_type_identificator import TypeIdentificator


class AronTypes:
    INT = "_ARON_INT"
    LONG = "_ARON_LONG"
    FLOAT = "_ARON_FLOAT"
    DOUBLE = "_ARON_DOUBLE"
    BOOL = "_ARON_BOOL"
    STRING = "_ARON_STRING"
    LIST = "_ARON_LIST"
    QUATERNION = "_ARON_QUATERNION"
    DICT = "_ARON_DICT"
    MATRIX = "_ARON_MATRIX"
    ENUM = "_ARON_INT_ENUM"
    OBJECT = "_ARON_OBJECT"


class AronOptionals:
    NONE = "type::maybe::NONE"
    OPTIONAL = "type::maybe::OPTIONAL"


class FluxioTypeIdentifier(str, Enum):
    BASIC = "BASIC"
    GENERIC = "GENERIC"
    DOUBLE_GENERIC = "DOUBLE_GENERIC"
    OBJECT = "OBJECT"
    ENUM = "ENUM"


@dc.dataclass
class FluxioBasicType:
    origin: TypeIdentificator
    name: str
    description: str
    type_identifier: FluxioTypeIdentifier
    optional: bool
    default_value: str | None

    def __init__(
        self,
        name: str,
        origin=TypeIdentificator("", ""),
        description: str = "",
        type_identifier: FluxioTypeIdentifier = FluxioTypeIdentifier.BASIC,
        optional: bool = False,
        default_value: str | None = None,
    ):
        self.origin = origin
        self.name = name
        self.description = description
        self.type_identifier = type_identifier
        self.optional = optional
        self.default_value = default_value


@dc.dataclass
class FluxioGenericType(FluxioBasicType):
    generic: FluxioBasicType
    max_number_of_elements: int

    def __init__(
        self,
        name: str,
        generic: FluxioBasicType,
        origin=TypeIdentificator("", ""),
        description: str = "",
        type_identifier: FluxioTypeIdentifier = FluxioTypeIdentifier.GENERIC,
        default_value: str = "",
        optional: bool = False,
        max_number_of_elements: int = 0,
    ):
        super().__init__(
            origin=origin,
            name=name,
            description=description,
            optional=optional,
            default_value=default_value,
            type_identifier=type_identifier,
        )
        self.generic = generic
        self.max_number_of_elements = max_number_of_elements


@dc.dataclass
class FluxioDoubleGenericType(FluxioBasicType):
    generic1: FluxioBasicType
    generic2: FluxioBasicType
    max_number_of_elements: int

    def __init__(
        self,
        name: str,
        generic1: FluxioBasicType,
        generic2: FluxioBasicType,
        origin=TypeIdentificator("", ""),
        description: str = "",
        type_identifier: FluxioTypeIdentifier = FluxioTypeIdentifier.DOUBLE_GENERIC,
        default_value: str = "",
        optional: bool = False,
        max_number_of_elements: int = 0,
    ):
        super().__init__(
            origin=origin,
            name=name,
            description=description,
            optional=optional,
            default_value=default_value,
            type_identifier=type_identifier,
        )
        self.generic1 = generic1
        self.generic2 = generic2
        self.max_number_of_elements = max_number_of_elements


@dc.dataclass
class FluxioObjectAttribute:
    name: str
    type: FluxioBasicType


@dc.dataclass
class FluxioObjectType(FluxioBasicType):
    attributes: list[FluxioObjectAttribute]
    parent: Optional["FluxioObjectType"]

    def __init__(
        self,
        name: str,
        origin=TypeIdentificator("", ""),
        description: str = "",
        type_identifier: FluxioTypeIdentifier = FluxioTypeIdentifier.OBJECT,
        optional: bool = False,
        default_value: str | None = None,
        parent: Optional["FluxioObjectType"] = None,
    ):
        super().__init__(
            origin=origin,
            name=name,
            description=description,
            optional=optional,
            default_value=default_value,
            type_identifier=type_identifier,
        )
        self.parent = parent
        self.attributes = []

    def add_attribute(self, name: str, type: FluxioBasicType):
        self.attributes.append(FluxioObjectAttribute(name=name, type=type))


@dc.dataclass
class FluxioEnumType(FluxioBasicType):
    attributes: dict[str, int]

    def __init__(
        self,
        name: str,
        origin=TypeIdentificator("", ""),
        description: str = "",
        type_identifier: FluxioTypeIdentifier = FluxioTypeIdentifier.ENUM,
        optional: bool = False,
        default_value: str | None = None,
    ):
        super().__init__(
            origin=origin,
            name=name,
            description=description,
            optional=optional,
            default_value=default_value,
            type_identifier=type_identifier,
        )
        self.attributes = {}

    def add_attribute(self, name: str, value: int):
        self.attributes[name] = value
