import dataclasses as dc

from armarx_skills.manager import dto
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)
from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_skills.manager.fluxio_data import FluxioBaseData
from armarx_skills.manager.fluxio_data_aron_converter import FluxioDataAronConverter


@dc.dataclass
class Value:
    """
    A class representing a value with a profile ID and content.

    Attributes:
    profileId (Identificator): The profile ID associated with the value.
    content (FluxioData): The content of the value.
    """

    profileId: Identificator
    content: FluxioBaseData


class FluxioValueConv(IceConverter):
    """
    A class for converting values to and from Fluxio format.

    This class inherits from IceConverter and provides methods for converting values to and from Fluxio format.

    Attributes:
    - None

    Methods:
    - __init__(): Initializes the FluxioValueConv object.
    - _import_dto(): Imports the DTO class for the converter.
    - _from_ice(): Converts the DTO to the Value object.
    - _to_ice(): Converts the Value object to the DTO.

    """

    def __init__(self):
        """
        Initializes the FluxioValueConv object.

        Parameters:
        - None

        Return:
        - None
        """
        super().__init__()

    @classmethod
    def _import_dto(cls):
        """
        This method is a class method that returns the FluxioValue data transfer object (DTO).

        Return:
        dto.FluxioValue: The FluxioValue data transfer object.

        Raises:
        None
        """
        return dto.FluxioValue

    def _from_ice(self, dt: dto.FluxioValue) -> Value:
        """
        Converts a dto.FluxioValue object to a Value object.

        Parameters:
        dt (dto.FluxioValue): The dto.FluxioValue object to convert.

        Return:
        Value: The converted Value object.

        Raises:
        None
        """
        return Value(
            profileId=FluxioIdentificatorConv().from_ice(dt.profileId),
            content=FluxioDataAronConverter.aron_data_to_fluxio_data(dt.content),
        )

    def _to_ice(self, bo: Value) -> dto.FluxioValue:
        """
        Converts a Value object to a FluxioValue object.

        Parameters:
        bo (Value): The Value object to be converted.

        Return:
        dto.FluxioValue: The converted FluxioValue object.

        Raises:
        None
        """
        return dto.FluxioValue(
            profileId=FluxioIdentificatorConv().to_ice(bo.profileId),
            content=FluxioDataAronConverter.fluxio_data_to_aron(bo.content),
        )
