import Ice

from typing import List, Optional, Callable, Dict

from armarx_core import ice_manager
from armarx_core.reconnecting_proxy_client import ReconnectingProxyClient

from armarx_memory.aron.aron_ice_types import AronDataIceTypes, AronTypeIceTypes

from armarx_skills.core import execution_status as es
from armarx_skills.core.execution_status import ExecutionStatus

from armarx_skills.provider import abort_skill_result as asr
from armarx_skills.provider import dto as provider_dto

from armarx_skills.manager import dti, dto
from armarx_skills.manager import errors
from armarx_skills.manager import provider_info as pi
from armarx_skills.manager import skill_execution_id as sei
from armarx_skills.manager import skill_execution_request as ser
from armarx_skills.manager import skill_status_update as ssu
from armarx_skills.manager.skill_id import SkillID, SkillIdConv, ProviderID
from armarx_skills.manager import skill_description as sd

from armarx_skills.manager.fluxio_skill_status_update import (
    SkillStatusUpdate,
    FluxioSkillStatusUpdateConv,
)
from armarx_skills.manager.fluxio_skill import Skill, FluxioSkillConv
from armarx_skills.manager.fluxio_value import Value, FluxioValueConv
from armarx_skills.manager.fluxio_parameter import Parameter, FluxioParameterConv
from armarx_skills.manager.fluxio_profile import Profile, FluxioProfileConv
from armarx_skills.manager.fluxio_provider import Provider, FluxioProviderConv
from armarx_skills.manager.fluxio_exception import FluxioException, FluxioExceptionConv
from armarx_skills.manager.fluxio_identificator import (
    Identificator,
    FluxioIdentificatorConv,
)
from armarx_skills.manager.fluxio_type_identificator import (
    TypeIdentificator,
    FluxioTypeIdentificatorConv,
)
from armarx_skills.manager.fluxio_type_aron_converter import FluxioTypeAronConverter
from armarx_skills.manager.fluxio_data_aron_converter import FluxioDataAronConverter
from armarx_skills.manager.fluxio_types import FluxioObjectType, FluxioTypeIdentifier


def fluxio_wrapper(self, func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except dto.FluxioException as e:
            raise self.fluxio_exception_converter.from_ice(e)

    return wrapper


class SkillManager:
    DEFAULT_ICE_OBJECT_NAME = "SkillMemory"

    def __init__(
        self,
        proxy: dti.SkillManagerInterfacePrx = None,
    ):
        self.proxy = proxy

        self.skill_id_conv = SkillIdConv()
        self.provider_info_conv = pi.ProviderInfoConv()
        self.skill_execution_id_conv = sei.SkillExecutionIDConv()
        self.skill_execution_request_conv = ser.SkillExecutionRequestConv()
        self.skill_status_update_conv = ssu.SkillStatusUpdateConv()
        self.skill_description_conv = sd.SkillDescriptionConv()
        self.skill_description_map_conv = sd.SkillDescriptionMapConv()
        self.abort_skill_result_conv = asr.AbortSkillResultConv()
        self.fluxio_skill_conv = FluxioSkillConv()
        self.fluxio_profile_conv = FluxioProfileConv()
        self.fluxio_value_conv = FluxioValueConv()
        self.fluxio_provider_conv = FluxioProviderConv()
        self.fluxio_skill_status_update_conv = FluxioSkillStatusUpdateConv()
        self.fluxio_exception_converter = FluxioExceptionConv()
        self.fluxio_identificator_conv = FluxioIdentificatorConv()
        self.fluxio_type_identificator_conv = FluxioTypeIdentificatorConv()
        self.fluxio_parameter_conv = FluxioParameterConv()

    def __getattribute__(self, name):
        attr = object.__getattribute__(self, name)
        if callable(attr):
            return fluxio_wrapper(self, attr)
        return attr

    @classmethod
    def wait_for_manager(
        cls,
        name=DEFAULT_ICE_OBJECT_NAME,
    ) -> "SkillManager":
        return cls(proxy=ice_manager.wait_for_proxy(dti.SkillManagerInterfacePrx, name))

    @classmethod
    def get_manager(
        cls,
        name=DEFAULT_ICE_OBJECT_NAME,
    ) -> Optional["SkillManager"]:
        try:
            return cls(proxy=ice_manager.get_proxy(dti.SkillManagerInterfacePrx, name))
        except Ice.ConnectionRefusedException:
            return None

    ### Fluxio related methods
    # Everything skill related

    def get_skills(
        self,
    ) -> List[Skill]:
        skills_ice = self.get_skills_ice()
        skills = []

        for s in skills_ice:
            skill = self.fluxio_skill_conv.from_ice(s)
            skills.append(skill)

        return skills

    def get_skills_ice(self) -> dto.FluxioSkillList:
        return self.proxy.getSkillList()

    def get_skill(
        self,
        skill_id: str,
    ) -> Skill | None:
        skill_ice = self.get_skill_ice(skill_id)

        if skill_ice is None:
            return None

        return self.fluxio_skill_conv.from_ice(skill_ice)

    def get_skill_ice(self, skill_id: str) -> Optional[dto.FluxioSkill]:
        return self._ice_unset_to_none(self.proxy.getSkill(id=skill_id))

    def update_skill(
        self,
        user_id: str,
        skill: Skill,
    ) -> None:
        skill_ice = self.fluxio_skill_conv.to_ice(skill)
        return self.update_skill_ice(user_id=user_id, skill=skill_ice)

    def update_skill_ice(self, user_id: str, skill: dto.FluxioSkill) -> None:
        return self.proxy.updateSkill(userId=user_id, skill=skill)
    
    def update_skill_parameter_values(
            self,
            user_id: str,
            skill_id: str,
            parameter_id: str,
            values: List[Value]
    ) -> None:
        values_ice = []
        for v in values:
            values_ice.append(self.fluxio_value_conv.to_ice(v))
        return self.update_skill_parameter_values_ice(
            user_id=user_id, skill_id=skill_id, parameter_id=parameter_id, values=values_ice
        )

    def update_skill_parameter_values_ice(
            self,
            user_id: str,
            skill_id: str,
            parameter_id: str,
            values: dto.FluxioValueList
    ) -> None:
        return self.proxy.updateSkillParameterValues(
            userId=user_id, skillId=skill_id, parameterId=parameter_id, values=values
        )

    def delete_skill(
        self,
        skill_id: str,
        user_id: str,
        dry_run: bool,
    ) -> List[Identificator] | None:
        affected_skills_ice = self.delete_skill_ice(
            skill_id=skill_id, user_id=user_id, dry_run=dry_run
        )
        affected_skills = []

        if affected_skills_ice is None:
            return None

        for res in affected_skills_ice:
            affected_skills.append(self.fluxio_identificator_conv.from_ice(res))

        return affected_skills

    def delete_skill_ice(
        self, skill_id: str, user_id: str, dry_run: bool
    ) -> Optional[dto.FluxioIdentificatorList]:
        return self.proxy.deleteSkill(skillId=skill_id, userId=user_id, dryRun=dry_run)

    def get_skill_mutex(
        self,
        skill_id: str,
        user_id: str,
    ) -> bool:
        return self.get_skill_mutex_ice(skill_id=skill_id, user_id=user_id)

    def get_skill_mutex_ice(self, skill_id: str, user_id: str) -> bool:
        return self.proxy.getSkillMutex(skillId=skill_id, userId=user_id)

    def delete_skill_mutex(
        self,
        skill_id: str,
        user_id: str,
    ) -> None:
        return self.delete_skill_mutex_ice(skill_id=skill_id, user_id=user_id)

    def delete_skill_mutex_ice(self, skill_id: str, user_id: str) -> None:
        return self.proxy.deleteSkillMutex(skillId=skill_id, userId=user_id)

    def update_skill_parameter(
        self,
        user_id: str,
        skill_id: str,
        parameter: Parameter,
        dry_run: bool,
    ) -> List[Identificator] | None:
        parameter_ice = self.fluxio_parameter_conv.to_ice(parameter)
        return self.update_skill_parameter_ice(
            user_id=user_id, skill_id=skill_id, parameter=parameter_ice,dry_run=dry_run
        )

    def update_skill_parameter_ice(
        self, user_id: str, skill_id: str, parameter: dto.FluxioParameter, dry_run: bool
    ) -> Optional[dto.FluxioIdentificatorList]:
        return self.proxy.updateSkillParameter(
            userId=user_id, skillId=skill_id, parameter=parameter, dryRun=dry_run
        )
    
    def delete_skill_parameter(
        self,
        user_id: str,
        skill_id: str,
        parameter_id: str,
        dry_run: bool,
    ) -> List[Identificator] | None:
        return self.delete_skill_parameter_ice(
            user_id=user_id, skill_id=skill_id, parameter_id=parameter_id,dry_run=dry_run
        )

    def delete_skill_parameter_ice(
        self, user_id: str, skill_id: str, parameter_id: str, dry_run: bool
    ) -> Optional[dto.FluxioIdentificatorList]:
        return self.proxy.deleteSkillParameter(
            userId=user_id, skillId=skill_id, parameterId=parameter_id, dryRun=dry_run
        )

    # Everything profile related
    def get_profiles(
        self,
    ) -> List[Profile]:
        profiles_ice = self.get_profiles_ice()
        profiles = []

        for p in profiles_ice:
            profile = self.fluxio_profile_conv.from_ice(p)
            profiles.append(profile)

        # sort profiles by dependencies
        save_break = 0
        prof_len = len(profiles)
        sorted_profiles = []
        while len(profiles) > 0:
            # save break with pover of 2
            if save_break == 2 ** prof_len:
                break
            for p in profiles:
                if p.parentId is None or p.parentId.id in [
                    sp.id for sp in sorted_profiles
                ]:
                    sorted_profiles.append(p)
                    profiles.remove(p)
            save_break += 1

        return sorted_profiles

    def get_profiles_ice(self) -> dto.FluxioProfileList:
        return self.proxy.getProfileList()

    def get_profile(
        self,
        profile_id: str,
    ) -> Profile | None:
        profile_ice = self.get_profile_ice(profile_id=profile_id)

        if profile_ice is None:
            return None

        return self.fluxio_profile_conv.from_ice(profile_ice)

    def get_profile_ice(self, profile_id: str) -> Optional[dto.FluxioProfile]:
        return self._ice_unset_to_none(self.proxy.getProfile(id=profile_id))

    def create_profile(
        self,
        profile: Profile,
    ) -> Profile:
        profile_ice = self.fluxio_profile_conv.to_ice(profile)
        profile_with_id_ice = self.create_profile_ice(profile=profile_ice)

        return self.fluxio_profile_conv.from_ice(profile_with_id_ice)

    def create_profile_ice(self, profile: dto.FluxioProfile) -> dto.FluxioProfile:
        return self.proxy.createProfile(profile=profile)

    def update_profile(
        self,
        profile: Profile,
    ) -> None:
        profile_ice = self.fluxio_profile_conv.to_ice(profile)
        return self.update_profile_ice(profile=profile_ice)

    def update_profile_ice(self, profile: dto.FluxioProfile) -> None:
        return self.proxy.updateProfile(profile=profile)

    # Everything provider related
    def get_providers(
        self,
    ) -> List[Provider]:
        providers_ice = self.get_providers_ice()
        providers = []

        for p in providers_ice:
            provider = self.fluxio_provider_conv.from_ice(p)
            providers.append(provider)

        return providers

    def get_providers_ice(self) -> dto.FluxioProviderList:
        return self.proxy.getProviderList()

    def get_provider(
        self,
        provider_id: str,
    ) -> Provider | None:
        provider_ice = self.get_provider_ice(provider_id=provider_id)

        if provider_ice is None:
            return None

        return self.fluxio_provider_conv.from_ice(provider_ice)

    def get_provider_ice(self, provider_id: str) -> Optional[dto.FluxioProvider]:
        return self._ice_unset_to_none(self.proxy.getProvider(id=provider_id))

    def get_skills_of_provider(
        self,
        provider_id: str,
    ) -> List[Skill] | None:
        skills_ice = self.get_skills_of_provider_ice(provider_id=provider_id)

        if skills_ice is None:
            return None

        skills = []

        for s in skills_ice:
            skill = self.fluxio_skill_conv.from_ice(s)
            skills.append(skill)

        return skills

    def get_skills_of_provider_ice(
        self, provider_id: str
    ) -> Optional[dto.FluxioSkillList]:
        list = self.proxy.getSkillsOfProvider(id=provider_id)

        return self._ice_unset_to_none(list)

    def add_new_skill_to_provider(
        self,
        user_id: str,
        provider_id: str,
        skill: Skill,
    ) -> Skill | None:
        skill_ice = self.fluxio_skill_conv.to_ice(skill)
        skill_with_id_ice = self.add_new_skill_to_provider_ice(
            user_id=user_id, provider_id=provider_id, skill=skill_ice
        )

        if skill_with_id_ice is None:
            return None

        return self.fluxio_skill_conv.from_ice(skill_with_id_ice)

    def add_new_skill_to_provider_ice(
        self, user_id: str, provider_id: str, skill: dto.FluxioSkill
    ) -> Optional[dto.FluxioSkill]:
        return self._ice_unset_to_none(
            self.proxy.addSkillToProvider(
                userId=user_id, providerId=provider_id, skill=skill
            )
        )

    def get_types(self) -> AronTypeIceTypes.Object:
        return self.proxy.getTypes()

    def get_types_fluxio(self) -> FluxioObjectType:
        types = self.get_types()
        res = FluxioTypeAronConverter.aron_type_to_fluxio_type(types)
        self.fix_fluxio_types_origin(res)
        return res

    def fix_fluxio_types_origin(self, fluxio_types: FluxioObjectType) -> None:
        for attr in fluxio_types.attributes:
            skillId = attr.name
            if attr.type.type_identifier != FluxioTypeIdentifier.OBJECT:
                return
            if isinstance(attr.type, FluxioObjectType):
                for attr2 in attr.type.attributes:
                    parameterId = attr2.name
                    attr2.type.origin = TypeIdentificator(skillId, parameterId)

    def get_test_data(self) -> AronDataIceTypes.Dict:
        res: AronDataIceTypes.Dict = self.proxy.getTestData()
        ret = FluxioDataAronConverter.aron_data_to_fluxio_data(res)
        return ret

    #### End of Fluxio related methods

    def add_provider(
        self,
        provider_info: pi.ProviderInfo,
    ) -> None:
        provider_info_ice = self.provider_info_conv.to_ice(provider_info)
        return self.add_provider_ice(provider_info_ice)

    def add_provider_ice(
        self,
        provider_info: dto.ProviderInfo,
    ) -> None:
        return self.proxy.addProvider(provider_info)

    def remove_provider(
        self,
        provider_info: pi.ProviderInfo,
    ) -> None:
        provider_info_ice = self.provider_info_conv.to_ice(provider_info)
        return self.remove_provider_ice(provider_info_ice)

    def remove_provider_ice(
        self,
        provider_info: dto.ProviderInfo,
    ) -> None:
        return self.proxy.removeProvider(providerInfo=provider_info)

    def get_skill_description(
        self,
        skill_id: SkillID,
    ) -> Optional[sd.SkillDescription]:
        skill_id_ice = self.skill_id_conv.to_ice(skill_id)
        description_ice = self.get_skill_description_ice(skill_id=skill_id_ice)

        if description_ice is None:
            return None
        else:
            description = self.skill_description_conv.from_ice(description_ice)
            return description

    def get_skill_description_ice(
        self,
        skill_id: dto.SkillID,
    ) -> Optional[dto.SkillDescription]:
        return self._ice_unset_to_none(self.proxy.getSkillDescription(skillId=skill_id))

    def get_skill_descriptions(
        self,
    ) -> sd.SkillDescriptionMap:
        descriptions_ice = self.get_skill_descriptions_ice()
        descriptions = self.skill_description_map_conv.from_ice(descriptions_ice)
        return descriptions

    def get_skill_descriptions_ice(
        self,
    ) -> dto.SkillDescriptionMap:
        return self.proxy.getSkillDescriptions()

    def get_fluxio_skill_execution_status(
        self,
        execution_id: str,
    ) -> List[SkillStatusUpdate] | None:
        status_list_ice = self.get_fluxio_skill_execution_status_ice(
            execution_id=execution_id
        )
        if status_list_ice is None:
            return None
        status_list = []
        for status_ice in status_list_ice:
            status = self.fluxio_skill_status_update_conv.from_ice(status_ice)
            status_list.append(status)

        return status_list

    def get_fluxio_skill_execution_status_ice(
        self,
        execution_id: str,
    ) -> Optional[dto.FluxioSkillStatusUpdateList]:
        return self.proxy.getFluxioSkillExecutionStatus(executionId=execution_id)

    def get_skill_execution_status(
        self,
        execution_id: sei.SkillExecutionID,
    ) -> Optional[ssu.SkillStatusUpdate]:
        execution_id_ice = self.skill_execution_id_conv.to_ice(execution_id)
        status_ice = self.get_skill_execution_status_ice(execution_id_ice)

        if status_ice is None:
            return None
        else:
            status = self.skill_status_update_conv.from_ice(status_ice)
            return status

    def get_skill_execution_status_ice(
        self,
        execution_id: dto.SkillExecutionID,
    ) -> Optional[dto.SkillStatusUpdate]:
        return self._ice_unset_to_none(self.proxy.getSkillExecutionStatus(execution_id))

    def get_skill_execution_statuses(
        self,
    ) -> ssu.SkillStatusUpdateMap:
        updates_ice = self.get_skill_execution_statuses_ice()
        updates = {
            self.skill_execution_id_conv.from_ice(
                k
            ): self.skill_status_update_conv.from_ice(v)
            for k, v in updates_ice.items()
        }
        return updates

    def get_skill_execution_statuses_ice(
        self,
    ) -> dto.SkillStatusUpdateMap:
        return self.proxy.getSkillExecutionStatuses()

    def execute_skill(
            self,
            request: ser.SkillExecutionRequest,
            asynchronous=False,
            log=None,
            dry=False,
    ) -> Optional[ssu.SkillStatusUpdate]:
        # Return is optional because returning None when dry == True.

        if log is not None:
            msg = "[DRY RUN] " if dry else ""
            msg += f"Executing skill {request.skill_id} with parameters: "
            for k, v in request.parameters.items():
                msg += f"\n- '{k}': {v}"
            log.info(msg)

        request_ice = self.skill_execution_request_conv.to_ice(request)

        if dry:
            return None

        if not asynchronous:
            update_ice = self.execute_skill_ice(request=request_ice)
            update: ssu.SkillStatusUpdate = self.skill_status_update_conv.from_ice(update_ice)
        else:
            skill_execution_id = self.execute_skill_async_ice(request=request_ice)
            update: ssu.SkillStatusUpdate = ssu.SkillStatusUpdate(sei.SkillExecutionIDConv().from_ice(skill_execution_id), {}, None,
                                                                  ExecutionStatus.Preparing, {})

        if log is not None:
            log.info(
                f"Skill {request.skill_id} terminated with status '{update.status.name}'."
            )

        return update

    def execute_skill_based_on_default_parameters(
            self,
            executor_name: str,
            skill_id: SkillID,
            parameters_function: Callable[[Dict], None],
            asynchronous=False,
            log=None,
            dry=False

    ) -> ssu.SkillStatusUpdate:
        """
        Execute a skill based on its default parameters.

        parameters_function is a function that edits the default parameters
        before executing the skill.
        """
        descriptions = self.get_skill_descriptions()
        try:
            description = descriptions[skill_id]
        except KeyError as e:
            raise errors.SkillNotFoundError(skill_id=skill_id)

        parameters = description.root_profile_defaults

        parameters_function(parameters)

        return self.execute_skill(ser.SkillExecutionRequest(
            executor_name=executor_name,
            skill_id=skill_id,
            parameters=parameters),
            asynchronous=asynchronous,
            log=log,
            dry=dry)

    def execute_skill_with_default_parameters(
        self,
        executor_name: str,
        skill_id: SkillID,
        log=None,
        dry=False,
    ) -> ssu.SkillStatusUpdate:
        """
        Execute a skill with its default parameters.
        """
        return self.execute_skill_based_on_default_parameters(
            executor_name=executor_name,
            skill_id=skill_id,
            parameters_function=lambda parameters: None,
            log=log,
            dry=dry,
        )

    def execute_skill_fluxio(
        self,
        skill_id: str,
        profile_id: str,
    ) -> str | None:
        execution_id = self.execute_skill_fluxio_ice(skill_id=skill_id, profile_id=profile_id)

        if execution_id is None:
            return None

        return execution_id

    def execute_skill_fluxio_ice(self, skill_id: str, profile_id: str) -> Optional[str]:
        return self.proxy.executeFluxioSkill(skillId=skill_id, profileId=profile_id)

    def abort_skill_fluxio(
        self,
        execution_id: str,
    ):
        self.abort_skill_fluxio_ice(execution_id=execution_id)

    def abort_skill_fluxio_ice(self, execution_id: str):
        self.proxy.abortFluxioSkill(executionId=execution_id)

    def execute_skill_ice(
        self,
        request: dto.SkillExecutionRequest,
    ) -> dto.SkillStatusUpdate:
        return self.proxy.executeSkill(skillExecutionRequest=request)

    def execute_skill_async_ice(
        self,
        request: dto.SkillExecutionRequest,
    ) -> dto.SkillExecutionID:
        return self.proxy.executeSkillAsync(skillExecutionRequest=request)

    def update_skill_parameters_ice(
        self,
        execution_id: dto.SkillExecutionID,
        parameters: AronDataIceTypes.Dict,
    ) -> dto.SkillExecutionID:
        return self.proxy.updateSkillParameters(
            executionId=execution_id,
            parameters=parameters,
        )

    def abort_skill(
        self,
        skill_execution_id: sei.SkillExecutionID,
        blocking=True,
    ) -> asr.AbortSkillResult:
        skill_execution_id_ice = self.skill_execution_id_conv.to_ice(skill_execution_id)

        abort_result_ice = self.abort_skill_ice(
            execution_id=skill_execution_id_ice,
            blocking=blocking,
        )

        abort_result = self.abort_skill_result_conv.from_ice(abort_result_ice)
        return abort_result

    def abort_skill_async(
        self,
        skill_execution_id: sei.SkillExecutionID,
    ) -> asr.AbortSkillResult:
        skill_execution_id_ice = self.skill_execution_id_conv.to_ice(skill_execution_id)

        abort_result_ice = self.abort_skill_async_ice(
            execution_id=skill_execution_id_ice,
        )

        abort_result = self.abort_skill_result_conv.from_ice(abort_result_ice)
        return abort_result

    def abort_skill_ice(
        self,
        execution_id: dto.SkillExecutionID,
        blocking=True,
    ) -> provider_dto.AbortSkillResult:
        if blocking:
            return self.proxy.abortSkill(executionId=execution_id)
        else:
            return self.abort_skill_async_ice(execution_id=execution_id)

    def abort_skill_async_ice(
        self,
        execution_id: dto.SkillExecutionID,
    ) -> provider_dto.AbortSkillResult:
        return self.proxy.abortSkillAsync(executionId=execution_id)

    def abort_all_non_terminated_skills(
        self,
        abort_statuses=(
          es.ExecutionStatus.Constructing,
          es.ExecutionStatus.Initializing,
          es.ExecutionStatus.Preparing,
          es.ExecutionStatus.Running,
        ),
        blocking=True,
        log=None,
    ) -> Dict[sei.SkillExecutionID, asr.AbortSkillResult]:
        # temporary fix until RobotAPI is merged
        """execution_ids_to_abort = list()

        execution_status_map = self.get_skill_execution_statuses()

        for execution_id, execution_status in execution_status_map.items():
            if execution_status.status in abort_statuses:
                log.info(f"{execution_id} RUNNING")
                execution_ids_to_abort.append(execution_id)
            else:
                log.info(f"{execution_id} not running")

        results = dict()
        for execution_id in execution_ids_to_abort:
            if log is not None:
                log.info(f"Aborting skill execution {execution_id} ...")
            result = self.abort_skill(
                skill_execution_id=execution_id, blocking=blocking
            )
            results[execution_id] = result

        return results"""

        return self.proxy.abortAllSkillsAsync()

    @staticmethod
    def _ice_unset_to_none(optional):
        return None if optional == Ice.Unset else optional


class ReconnectingSkillManager(ReconnectingProxyClient):
    def __init__(
        self,
        name=SkillManager.DEFAULT_ICE_OBJECT_NAME,
    ):
        super().__init__()
        self.name = name

    def get(self, wait=True, log=None, logger=None) -> Optional[SkillManager]:
        if logger is not None:
            logger.info("The parameter 'logger' is deprecated. Use 'log' instead.")
            if log is None:
                log = logger
            else:
                raise ValueError(
                    "The parameter 'logger' is deprecated. Use 'log' instead."
                )

        return self._get(wait=wait, log=log)

    def _get_client(self, log=None) -> Optional[SkillManager]:
        if log is not None:
            log.info(f"Waiting for skill manager '{self.name}' ...")

        client = SkillManager.get_manager(name=self.name)

        if log is not None:
            log.info(f"Connected to skill manager '{self.name}'.")

        return client

    def _wait_for_client(self, log=None) -> SkillManager:
        if log is not None:
            log.info(f"Getting skill manager '{self.name}' ...")

        client = SkillManager.wait_for_manager(name=self.name)

        if log is not None:
            if client is None:
                log.info(f"Failed to connect to skill manager '{self.name}'.")
            else:
                log.info(f"Connected to skill manager '{self.name}'.")

        return client


if __name__ == "__main__":

    def test_main():
        manager = SkillManager.wait_for_manager()
        manager.get_skill_execution_statuses()

    test_main()
