import typing as ty

import dataclasses as dc

from armarx_core.time.duration import Duration, DurationIceConv

from armarx_skills.provider import dto
from armarx_skills.provider.skill_id import SkillID, SkillIdConv

from armarx_memory.ice_conv.ice_converter import IceConverter
from armarx_memory.aron.conversion import pythonic_from_to_aron_ice


@dc.dataclass
class SkillDescription:
    skill_id: SkillID
    description: str
    timeout: Duration
    parameters_type: ty.Any  # armarx::aron::type::dto::AronObject
    result_type: ty.Any  # armarx::aron::type::dto::AronObject
    root_profile_defaults: ty.Dict[str, ty.Any]


SkillDescriptionMap = ty.Dict[SkillID, SkillDescription]


class SkillDescriptionConv(IceConverter):

    def __init__(self):
        super().__init__()
        self.skill_id_conv = SkillIdConv()
        self.duration_conv = DurationIceConv()

    @classmethod
    def _import_dto(cls):
        return dto.SkillDescription

    def _from_ice(self, dt: dto.SkillDescription) -> SkillDescription:
        return SkillDescription(
            skill_id=self.skill_id_conv.from_ice(dt.skillId),
            description=dt.description,
            timeout=self.duration_conv.from_ice(dt.timeout),
            parameters_type=dt.parametersType,
            result_type=dt.resultType,
            root_profile_defaults=pythonic_from_to_aron_ice.pythonic_from_aron_ice(
                dt.rootProfileDefaults,
            ),
        )

    def _to_ice(self, bo: SkillDescription) -> dto.SkillDescription:
        return dto.SkillDescription(
            skillId=self.skill_id_conv.to_ice(bo.skill_id),
            description=bo.description,
            timeout=self.duration_conv.to_ice(bo.timeout),
            parametersType=bo.parameters_type,
            resultType=bo.result_type,
            rootProfileDefaults=pythonic_from_to_aron_ice.pythonic_to_aron_ice(
                bo.root_profile_defaults,
            ),
        )


class SkillDescriptionMapConv(IceConverter):

    def __init__(self):
        super().__init__()
        self.skill_id_conv = SkillIdConv()
        self.skill_description_conv = SkillDescriptionConv()

    @classmethod
    def _import_dto(cls):
        return dto.SkillDescriptionMap

    def _from_ice(self, dt: dto.SkillDescriptionMap) -> SkillDescriptionMap:
        return {
            self.skill_id_conv.from_ice(k): self.skill_description_conv.from_ice(v)
            for k, v in dt.items(),
        }

    def _to_ice(self, bo: SkillDescriptionMap) -> dto.SkillDescriptionMap:
        return {
            self.skill_id_conv.to_ice(k): self.skill_description_conv.to_ice(v)
            for k, v in bo.items(),
        }
